
/********************************************************************************/
/*                        	* Frecuency-Counter code *                          */
/*                            By George A. Adamidis                             */
/*                            Ver 1.0 - March 17, 2009                          */ 
/********************************************************************************/
// All rights are reserved 


// Compiled using MPLAB-C18 V 8.10
// Loaded on a PIC18F2620 microcontroler
// All timing routines have been written for 4MHz master clock.
//--------------------------------------------------------------------------------

#include "Common.h"         /* General Definitions          */
#include <timers.h>         /* timer purpose routines       */
#include <p18f2620.h>        /* pic18f2620                    */
#include <p18cxxx.h>        /* TRISB and PORTB declarations */
#include <delays.h>         /* Delay purpose routines       */

/*functions declaration*/
void pause_counter(char mode);     			// Pause the counter through L,M or H-PIN - according to mode (mode=L,M or H)  
void start_counting (char mode);   			// Starts Counting through L,M or H-PIN - according to mode (mode=L,M or H)   
void resetcounter (void);      				// Resets the counter  (works only when counter is in pause mode)   
void counterup (void);         				// Counter increment by 1 (works only when counter is in pause mode)    
int prescaler(void);         				// Shifts out & returns, prescaler's internal counter value
char low_or_medium (void);	   				// Returns L or M if a low or medium frequency measurement must take place, respectively	
int external_counter (void);				// Returns the value of the external counter (return value is from 0 to 15)
extern char buttonpressed (void);

// variables - declaration
extern volatile unsigned long Overflow;
extern char mode;


/**************************************************************************/
// Pause the counter through L,M or H-PIN - according to mode (mode=L,M or H)  
void pause_counter  (char mode)
{
	switch (mode)
			{
				case 'L':
					L_PIN=off;
				break;

				case 'M':
					M_PIN=off;
				break;

				case 'H':
					H_PIN=off;
				break;
			}

}


/**************************************************************************/
 // Starts Counting through L,M or H-PIN - according to mode (mode=L,M or H) 
 void start_counting (char mode)   
{
		switch (mode)
			{
				case 'L':
					SH_PIN=on;  
					C_PIN=on;   
					L_PIN=on;
				break;

				case 'M':
					SH_PIN=on;  
					C_PIN=on;   
					M_PIN=on;
				break;

				case 'H':
					SH_PIN=on;  
					C_PIN=on;   
					H_PIN=on;
					
					// Some Calibration Delay
					Nop();                              // wait 1s for the counter to count
					Nop();   
				break;
			}
	
}


/**************************************************************************/
// Resets the counter  (works only when counter is in pause mode)
void resetcounter (void)      
{

// Reset external counter
RST_PIN=off;

// Resets Timer0 and starts the Timer 0 - interrupt, whitch takes place at timer's 0 overflow

OpenTimer0( TIMER_INT_ON &         // Interrupt enabled
T0_16BIT &                         // 16 BIT
T0_SOURCE_EXT &		    		   // External clock source (I/O pin)
T0_PS_1_256 &					   // 1:256 prescale
T0_EDGE_FALL);          		   // External clock on falling edge   
WriteTimer0(0);					   // Initializes Timer0 at 0.


INTCON2bits.TMR0IP=1;       // Sets TMR0 Overflow Interrupt Priority at High level 
RCONbits.IPEN=1;			// 1 = Enable priority levels on interrupts
INTCONbits.PEIE=0;			// Disables all low priority peripheral interrupts
INTCONbits.GIE=1;			// Enables all high priority interrupts
INTCONbits.TMR0IE=1;        // Enables the TMR0 overflow interrupt
Overflow=0; 				// Resets overflow variable

RST_PIN=on;
}


/**************************************************************************/
// Counter increment by 1 (works only when counter is in pause mode)
void counterup (void)        
{
//Creates one pulse at the counter strobe control pin - transition from high to low logic level
SH_PIN=on;
Nop();
SH_PIN=off;
Nop();
}


/**************************************************************************/
// Shifts out & returns, prescaler's internal counter value
 int prescaler(void)
{
int value=0;
int countr=0;
int i=0;
countr=ReadTimer0();       			// Reads TMR0 counter value and passes it to the countr variable
SH_PIN=off;
C_PIN=off;	
	while (countr==ReadTimer0()) 
		{
		counterup();
		i++;
		}  
value=256-i;
return value;
}  


/**************************************************************************/
// Returns L or M if a low or medium frequency measurement must take place, respectively
char low_or_medium (void)
{
	char i='L';
	int k;
	start_counting('M');	     // Starts the counter                   
	Delay1KTCYx(50);			 // wait 0.05s for the counter to count 
	pause_counter('M');				
	k=ReadTimer0();
	if (k>12) i='M';	
	resetcounter ();
	return i;
}		
		

/**************************************************************************/
// Returns the value of the external counter (return value is from 0 to 15)	
int external_counter (void)
{
	int a;
	a=0;
	if (D0_PIN) a=a+1;
	if (D1_PIN) a=a+2;
	if (D2_PIN) a=a+4;
	SH_PIN=on;
	C_PIN=on;
	if (D3_PIN) a=a+8;
	C_PIN=off;
	return a;	
}

			