
/********************************************************************************/
/*                        	* Frecuency-Counter code *                          */
/*                            By George A. Adamidis                             */
/*                            Ver 1.0 - March 17, 2009                          */ 
/********************************************************************************/
// All rights are reserved 


// Compiled using MPLAB-C18 V 8.10
// Loaded on a PIC18F2620 microcontroler
// All timing routines have been written for 4MHz master clock.
//--------------------------------------------------------------------------------

#include "Common.h"         /* General Definitions          */
#include <delays.h>         /* Delay purpose routines       */
#include <timers.h>         /* timer purpose routines       */
#include <xlcd.h>           /* LCD purpose routines         */
#include <p18cxxx.h>        /* TRISB and PORTB declarations */
#include <p18f2620.h>       /* for pic18f2620              */

/* Set configuration bits */ 
#pragma config OSC = XT   // set XT oscillator
#pragma config WDT = OFF  // disable watchdog timer
#pragma config LVP = OFF  // disable low voltage programming


/* Device turned off, if VDD<4,5V */ 
#pragma config BOREN = ON
#pragma config BORV = 2   


/*Code protection*/
#pragma config CP0 = ON
#pragma config CP1 = ON  


/*functions declaration*/
void timer_isr (void);         				//Interrupt routine, take place at TMR0's counter overflow
void freq (void);		   	   				// Finds current frequency, Max Hold frequency, Min Hold frequency  in dHz (values are stored in "frequency", "maxhold" & "minhold" variables.			
void displayfreq (unsigned long freq); 		// Displays frequency freq. - Affects digit[] array
void delay (char gate);		   				// Delay for 0.1, 1 or 10 sec for gate =0,1 or 2, respectively
extern void disst(void); 	   				// Displays current device state at LCD's second line,

extern char low_or_medium (void);     	// Returns L or M if a low or medium frequency measurement must take place, respectively
extern void pause_counter(char mode);   // Pause the counter through L,M or H-PIN - according to mode (mode=L,M or H)  
extern void start_counting (char mode); // Starts Counting through L,M or H-PIN - according to mode (mode=L,M or H)   
extern void resetcounter (void);        // Resets the counter  (works only when counter is in pause mode)   
extern char buttonpressed (void);
extern void action(unsigned char but);  // Perfoms the appropriate function if button "but" has been pressed (but - 1,2,3 or 4)
extern int prescaler(void); 
extern int external_counter (void);

// variables - declaration
unsigned long Overflow=0;      // Overflow = how many "overflows" took place on TMR0 counter during counting 
unsigned char button=0;        // button-->0,1,2 or 3 if no button, mode button, function button or units button has been pressed during counting, respectively
char mode=1;                   // mode=1 --> Normal counting mode (counts for 1sec), mode=0--> fast counting mode (counts for 0.1sec),mode=2--> Slow counting mode (counts for 10sec) 
char function=0;               // function=0 --> Realtime measuring, function=1 --> Max Hold measuring, function=2 --> Min Hold measuring 
char units=0;                  // units=0,1,2 --> Dsplays frequency in Hz, KHz, MHz units respectively
char channel=1;				   // channel=1 or 2 --> for channel A and channel B respectively
unsigned long frequency=0;     			// Current frequency - in dHz
unsigned long maxhold=0;	   			// Current Max Hold frequency
unsigned long minhold=19999999999;     	// Current Min Hold frequency in dHz
char digit[12]="           ";  			// Frequency digits - display 	
char led=0;                   		    // Current Led state (1 =on, 0=off)

// High priority interrupt vector
#pragma code InterruptVectorHigh = 0x08
void InterruptVectorHigh (void)                            
{
  _asm
  goto timer_isr	 //jump to interrupt routine
  _endasm
}

// High priority interrupt routine
#pragma code
#pragma interrupt timer_isr 
/*Interrupt routine, take place at TMR0's counter overflow*/
void timer_isr ()
{
	INTCONbits.TMR0IF = 0;      // Interrupt done - Interrupt bits set to 0
       Overflow++;              // 
}


/****************** Main body starts here *************************/
void main ()
{
	char meas;

	Delay10KTCYx(20);                   // wait 0.2s for peripherals to be initialized

	TRISB=0;                            // configure portB for output
	TRISC=15;                           // configure portC for input (RC0-RC3) and output (RC4-RC7)
	TRISA=23;                           // configure portA for input (RA0-RA2), (RA4) and output (RA3,RA5)
	ADCON1=15;                          // all digital.

	//Initializes LCD I/O pins used to control HD44780 LCD controller in 4bit interface and initializes LCD as a 5x7 multiple line.
	while( BusyXLCD() );                // waits until LCD not busy
	OpenXLCD(FOUR_BIT & LINES_5X7 );        
	
	while( BusyXLCD() );                // waits until LCD not busy
	WriteCmdXLCD( CURSOR_OFF );         // Cursor off on LCD
	
	while( BusyXLCD() );                // waits until LCD not busy 
	SetDDRamAddr(0x00);                 // Sets LCDs cursor on first position on first line 
	putrsXLCD("Frequency  Meter");      // Displays " FREQUENCY  METER" on LCD's first line    
	
	while( BusyXLCD() );                // waits until LCD not busy 
	SetDDRamAddr(0x40);                 // Sets LCDs cursor on first position on second line 
	putrsXLCD("Chania.TEI-Crete");      // Displays "Chania.TEI-Crete" on LCD's second line    
	LED=off;			                // Turns off Led
	led=off;
	Delay10KTCYx(200);                  // wait 2s 
	Delay10KTCYx(100);                  // wait 1s 
	disst();
	displayfreq(0);
	
	// infinitive loop starts here
	label:SH_PIN=on;             		// Sets the counter strobe control pin at high logic level 
	pause_counter  ('L');
	pause_counter  ('M');
	pause_counter  ('H');
	Delay1KTCYx(5);						// wait 5ms 
	resetcounter ();   
	Delay1KTCYx(5);			 			// wait 5ms 
	button=0;							// resets button variable
	Overflow=0;                         // resets overflow variable
	
	switch (channel)
	
			{
	
				case 1:
					meas=low_or_medium();
				break;
				
				case 2:
					meas='H';
				break;
			}
		
				
	switch (mode)               			// mode=0,1,2--> fast, normal, or slow counting mode respectively  
         {
           case 0:                          // Fast counting mode has been selected (counts for 0.1sec)         
				start_counting (meas);      // Starts the counter                   
				delay (0);					// wait 0.1 sec 
				pause_counter(meas);        // Stops counting
	       break;
          
	       case 1:                			// Normal counting mode has been selected (counts for 1sec)     
				start_counting (meas);      // Starts the counter                   
				delay (1);					// wait 1 sec 
				pause_counter(meas);        // Stops counting				
			break;

			case 2:                			// Slow counting mode has been selected (counts for 10sec)     
				start_counting (meas);      // Starts the counter                   
				delay (2);					// wait 10 sec 
				pause_counter(meas);        // Stops counting				
			break;

        }
	
	Delay1KTCYx(1);			 			 // wait 1ms 
	if (button>0) goto label;
		
	freq();								// Finds current frequency

	switch (function)
		{
			case 0:						// Displays Frequency	   	
	   			displayfreq(frequency);                          
			break;
	
			case 1: 					// Displays Max Hold frequency
 	   			displayfreq(maxhold);	
			break;
	
			case 2:						// Displays Min Hold frequency
	   			displayfreq(minhold);	
			break;
     	}  

	switch (led)  
		{
			case 0:
				LED=on;                  // Turns on Led
				led=on;
			break;

			case 1:
				LED=off;                 // Turns off Led
				led=off;		
			break;
		}

	goto label;						// infinitive loop ends here
}

/****************** Main body Ends here ******************************************************************/




 // Finds current frequency, Max Hold frequency, Min Hold frequency  in dHz or Hz (values are stored in "frequency", "maxhold" & "minhold" variables.			
void freq (void)
{
unsigned long buffer=0;
unsigned long buf=0;
unsigned long counter=0;
unsigned long counter2=0;
counter=ReadTimer0();							  // Reads current TMR0's (counter) value
buf=prescaler();								  // Reads the internal prescaler
counter2=external_counter();
buffer=(counter2)+(16*buf)+(4096*counter); 	      // Finds Counter's value

if (channel==1)
	{

		switch (mode)								  // Finds Frequency in dHz	
			{
				case 0:
					buffer=buffer*100;
				break;

				case 1:
					buffer=buffer*10;
				break;

			}
	}


if (channel==2)
	{

		switch (mode)								  // Finds Frequency in Hz	
			{
				case 0:
					buffer=buffer*10;
				break;

				case 2:
					buffer=buffer/10;
				break;

			}
		
		buffer=buffer*PRESC;					    // Taking into acount the external prescaler for channel 2

	}

frequency=buffer;								// Finds frequency
if (buffer<minhold) minhold=buffer;	      	    // Finds Min Hold frequency
if (minhold==0) minhold=buffer;					// Finds Min Hold frequency
if (buffer>maxhold)maxhold=buffer;			    // Finds Max Hold frequency

}		  



// Displays frequency freq. - Affects digit[] array
void displayfreq (unsigned long freq)
{
unsigned long d[12]="            ";
char *dig;
char i;
char j;
unsigned long fr;

fr=freq;

	// Convert to freq to ASCII

			d[11]=48+(freq%10);	
			freq=freq/10;
			d[10]=48+(freq%10);	
			freq=freq/10;
			d[9]=48+(freq%10);	
			freq=freq/10;
			d[8]=48+(freq%10);	
			freq=freq/10;
			d[7]=48+(freq%10);	
			freq=freq/10;
			d[6]=48+(freq%10);	
			freq=freq/10;
			d[5]=48+(freq%10);	
			freq=freq/10;
			d[4]=48+(freq%10);	
			freq=freq/10;
			d[3]=48+(freq%10);	
			freq=freq/10;
			d[2]=48+(freq%10);	
//			freq=freq/10;
//			d[1]=48+(freq%10);	
//			freq=freq/10;
//			d[0]=48+(freq%10);	

	
if (channel==1)
	{
	
	switch (units)			      			 // units=1,2,3 --> frequency in Hz, KHz, MHz units respectively			
		{
	        
	           	case 0:                       // frequency is going to be displayed in Hz units      
					digit[0]=d[3];     	
					digit[1]=d[4];     	
					digit[2]=d[5];     	
					digit[3]=d[6];     	
					digit[4]=d[7];     	
					digit[5]=d[8];     	
					digit[6]=d[9];     	
					digit[7]=d[10];     	
					digit[8]='.';  
					digit[9]=d[11];
					digit[10]=' ';
					digit[11]=0;
					


				break;                   
	          
			   	case 1:                      // frequency is going to be displayed in KHz units      
					digit[0]=d[3];     	
					digit[1]=d[4];     	
					digit[2]=d[5];     	
					digit[3]=d[6];     	
					digit[4]=d[7];     	
					digit[5]='.';     	
					digit[6]=d[8];     	
					digit[7]=d[9];     	
					digit[8]=d[10];  
					digit[9]=d[11];
					digit[10]=' ';
					digit[11]=0;


		          	
		        break;
			
			   	case 2:                     // frequency is going to be displayed in MHz units           
					digit[0]=d[3];     	
					digit[1]=d[4];     	
					digit[2]='.';     	
					digit[3]=d[5];     	
					digit[4]=d[6];     	
					digit[5]=d[7];     	
					digit[6]=d[8];     	
					digit[7]=d[9];     	
					digit[8]=d[10];  
					digit[9]=d[11];
					digit[10]=' ';
					digit[11]=0;


	
				break;
		}
		
		if (fr>999999999)Overflow++;	//Display overflow for frequency>99,9999999 MHz
	}

if (channel==2)
	{
		
	switch (units)			      				// units=1,2,3 --> frequency in Hz, KHz, MHz units respectively			
		{
	        
	           case 0:                       	// frequency is going to be displayed in Hz units      
					digit[0]=d[2];     	
					digit[1]=d[3];     	
					digit[2]=d[4];     	
					digit[3]=d[5];     	
					digit[4]=d[6];     	
					digit[5]=d[7];     	
					digit[6]=d[8];     	
					digit[7]=d[9];     	
					digit[8]=d[10];  
					digit[9]=d[11];
					digit[10]=' ';
					digit[11]=0;

				break;                   
	          
			   case 1:                      // frequency is going to be displayed in KHz units      
					digit[0]=d[2];     	
					digit[1]=d[3];     	
					digit[2]=d[4];     	
					digit[3]=d[5];     	
					digit[4]=d[6];     	
					digit[5]=d[7];     	
					digit[6]=d[8];     	
					digit[7]='.';     	
					digit[8]=d[9];  
					digit[9]=d[10];
					digit[10]=d[11];
					digit[11]=0;
		          	
		        break;
			
			   case 2:                     // frequency is going to be displayed in MHz units           
					digit[0]=d[2];     	
					digit[1]=d[3];     	
					digit[2]=d[4];     	
					digit[3]=d[5];     	
					digit[4]='.';     	
					digit[5]=d[6];     	
					digit[6]=d[7];     	
					digit[7]=d[8];	
					digit[8]=d[9];  
					digit[9]=d[10];
					digit[10]=d[11];
					digit[11]=0;
				break;
		}
		
		if (fr>1600000000)Overflow++;	 	//Display overflow for frequency>1.6 GHz
		
		if (fr<1000)						// Bug fix for 
			{	
					digit[0]=' ';     	
					digit[1]=' ';     	
					digit[2]=' ';     	
					digit[3]=' ';     	
					digit[4]=' ';     	
					digit[5]=' ';     	
					digit[6]=' ';     	
					digit[7]=' ';	
					digit[8]=' ';  
					digit[9]=' ';
					digit[10]='0';
					digit[11]=0;		

			}
	}


if (Overflow>0)
	{ 
		digit[0]=' ';
		digit[1]=' ';
		digit[2]='O';
		digit[3]='V';
		digit[4]='E';
		digit[5]='R';
		digit[6]='F';
		digit[7]='L';
		digit[8]='O';
		digit[9]='W';
		digit[10]=' ';
		digit[11]=0;
	}


// Remove zeroes before dot
i=0;
j=digit[i];

while ((j==48)&(i<12))
	{
		digit[i]=' ';
		i++;
		j=digit[i];
	}

if (j=='.') digit[i-1]='0';


dig=&digit[0];

while( BusyXLCD() );                // waits until LCD not busy 
SetDDRamAddr(0x00);                 // Sets LCDs cursor on first position on first line 

if (channel==1)
	{
		while( BusyXLCD() );               
		putrsXLCD("A ");       
	}

if (channel==2)
	{
	while( BusyXLCD() );              
		putrsXLCD("B ");       
	}

	while( BusyXLCD() );    
	putsXLCD(dig);	


if (Overflow==0)
		{
			switch (units)
				{
					case 0:
						while( BusyXLCD() );                
						putrsXLCD(" Hz");       			
					break;

					case 1:
						while( BusyXLCD() );              
						putrsXLCD("KHz");       			
					break;

					case 2:
						while( BusyXLCD() );               
						putrsXLCD("MHz");       			
					break;

				}

		}	 

if (Overflow>0)
	{
	while( BusyXLCD() );              
		putrsXLCD("    ");       
	}
  
}

/*******************************************************************/
// Delay for 0.1, 1 or 10 sec for gate =0,1 or 2, respectively
void delay (char gate)
{
	char i;

	switch (gate)
			{
				case 0:						 // 0.1s delay
					Delay10KTCYx(9);         // wait 0.09s  for the counter to count
					Delay1KTCYx(9);			 // wait 0.009s for the counter to count 
					Delay100TCYx(9);		 // wait 0.0009s for the counter to count
					Delay10TCYx(2);          // wait 0.00002s for the counter to count
			//		Nop();					 // wait 1s for the counter to count
					Nop();					 // wait 1s for the counter to count
					Nop();					 // wait 1s for the counter to count
					Nop();					 // wait 1s for the counter to count
					Nop();					 // wait 1s for the counter to count
					Nop();					 // wait 1s for the counter to count
					Nop();					 // wait 1s for the counter to count
					Nop();					 // wait 1s for the counter to count
					Nop();					 // wait 1s for the counter to count

					button=buttonpressed();              // Checks if any button is being pressed
					if (button!=0) 	action(button);     // If any button has been pressed, performs the appropriate action
											 	
				break;

				case 1:						// 1s delay
					
					for (i=1; i<4; i++)
						{
							Delay10KTCYx(25);                   // wait 0.25s for the counter to count
							button=buttonpressed();             // Checks if any button is being pressed
								if (button!=0) 
										{
											action(button);      // If any button has been pressed, performs the appropriate action
											return;
										}
						}
				
					Delay10KTCYx(24);                   // wait 0.24s for the counter to count
					button=buttonpressed();             // Checks if any button is being pressed
						if (button!=0) 
								{
									action(button);      // If any button has been pressed, performs the appropriate action
									return;
								}

					Delay1KTCYx(9);			 			// wait 0.009s for the counter to count 
					Delay100TCYx(8);                    // wait 0.0008s for the counter to count
					Delay10TCYx(8);                     // wait 0.00008s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count


					
					
				break;

				case 2:					// 10s delay		
					
					for (i=1; i<40; i++)
						{
							Delay10KTCYx(25);                   // wait 0.25s for the counter to count
							button=buttonpressed();             // Checks if any button is being pressed
								if (button!=0) 
									{
										action(button);      // If any button has been pressed, performs the appropriate action
										return;
									}
					
						}
				
					Delay10KTCYx(24);                   // wait 0.24s for the counter to count
					button=buttonpressed();             // Checks if any button is being pressed
						if (button!=0) 
								{
									action(button);      // If any button has been pressed, performs the appropriate action
									return;
								}
					Delay1KTCYx(8);			 			// wait 0.009s for the counter to count 
					Delay100TCYx(11);			 		// wait 0.0009s for the counter to count 
					Delay10TCYx(28);			 		// wait 0.00009s for the counter to count 
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
					Nop();                              // wait 1s for the counter to count
//					Nop();                              // wait 1s for the counter to count
//					Nop();                              // wait 1s for the counter to count
//					Nop();                              // wait 1s for the counter to count
//					Nop();                              // wait 1s for the counter to count	

				break;				

			}
}		     