//============================================================================
// Hierarchical state machine manager
//
// Hkbd.C : Exemple of a state machine definition (keyboard manager exemple)
//
// (c) Robert Lacoste 2013, 
// Based on ideas from Miro Samek's book "Practival UML statecharts in C/C++)"
//
// Use this software freely, but provided without any warranty or support
//============================================================================

#include <stdio.h>
#include <stdlib.h>
#include "Timer.h"
#include "Hkbd.h"
#include "Hcmd.h"

//----------------------------------------------------------------------------
// Applicative constant and variables
//----------------------------------------------------------------------------

#define CODELEN 4
#define CODEREF "12AC"

unsigned short Len;
char Code[CODELEN];


//----------------------------------------------------------------------------
// State machine initialisation
//----------------------------------------------------------------------------

void HkbdInit(void)
{
	HsmPriority[HSM_ID_Hkbd]=HSM_PRIORITY_LOW;
	HsmSetNewState(HSM_ID_Hkbd,&HkbdState_WaitingActivation);
}


//----------------------------------------------------------------------------
// State handlers (one per state)
//----------------------------------------------------------------------------

void HkbdState_WaitingActivation(const HSM_T_SIGNAL Signal)
{
	switch(Signal.SignalId)
	{
		case HSM_ON_ENTRY : 
			Len=0;
			break;
		case HSM_ON_EXIT : 
			TimerArm(TIMER_T1,HSM_ID_Hkbd,10000L);
			break;
		case Hkbd_KEY_ACTIVATE :
			printf("Enter code : \n");
			HsmSetNewState(HSM_ID_Hkbd,&HkbdState_WaitingDigits);
			break;
	}
}


void HkbdState_WaitingDigits(const HSM_T_SIGNAL Signal)
{
	BOOL SignalManaged=FALSE;	// Has a father state
	switch(Signal.SignalId)
	{
		case HSM_ON_ENTRY : 
			SignalManaged=TRUE;
			printf("(digit)");
			break;
		case HSM_ON_EXIT : 
			SignalManaged=TRUE;
			break;
		case Hkbd_KEY_ALPHAMODE :
			SignalManaged=TRUE;
			HsmSetNewState(HSM_ID_Hkbd,&HkbdState_WaitingAlpha);
			break;
		case Hkbd_KEY_DIGIT :
			if(Len<CODELEN) // Guard
			{
				SignalManaged=TRUE;
				putch(Code[Len++]=(unsigned char)Signal.SignalParameter+'0');
			}
			break;
	}
	
	// Call the father state
	if(!SignalManaged)
		(* HkbdState_MacroStateDigits)(Signal);
}


void HkbdState_WaitingAlpha (const HSM_T_SIGNAL Signal)
{
	BOOL SignalManaged=FALSE;	// Has a father state
	switch(Signal.SignalId)
	{
		case HSM_ON_ENTRY : 
			SignalManaged=TRUE;
			printf("(alpha)");
			break;
		case HSM_ON_EXIT : 
			SignalManaged=TRUE;
			break;
		case Hkbd_KEY_ALPHAMODE :
			SignalManaged=TRUE;
			HsmSetNewState(HSM_ID_Hkbd,&HkbdState_WaitingDigits);
			break;
		case Hkbd_KEY_DIGIT :
			if(Len<CODELEN)	// Guard
			{
				SignalManaged=TRUE;
				putch(Code[Len++]=(unsigned char)Signal.SignalParameter+'A');
				HsmSetNewState(HSM_ID_Hkbd,&HkbdState_WaitingDigits);
			}
			break;
	}
	
	// Call the father state
	if(!SignalManaged)
		(* HkbdState_MacroStateDigits)(Signal);
}


void HkbdState_MacroStateDigits (const HSM_T_SIGNAL Signal)
{
	switch(Signal.SignalId)
	{
		case TIMER_TIMEOUT_T1 :
			printf("Redo from start\n");
			HsmSetNewState(HSM_ID_Hkbd,&HkbdState_WaitingActivation);
			break;
		case Hkbd_KEY_DEL :
			if(Len>0)	// Guard
			{
				printf("(del)");
				Len--;
			}
			break;
		case Hkbd_KEY_ENTER :
			if((Len==CODELEN)&&(strncmp(Code,CODEREF,Len)==0))	// Guard
			{
				HsmSendMsg(HSM_ID_Hcmd,Hcmd_GIVE_SOME,0);
				HsmSetNewState(HSM_ID_Hkbd,&HkbdState_WaitingSafetyDelay);
			}
			else // Guard
			{
				printf("Sorry !\n");
				HsmSetNewState(HSM_ID_Hkbd,&HkbdState_WaitingActivation);
			}
			break;
	}
}


void HkbdState_WaitingSafetyDelay (const HSM_T_SIGNAL Signal)
{
	switch(Signal.SignalId)
	{
		case HSM_ON_ENTRY : 
			TimerArm(TIMER_T2,HSM_ID_Hkbd,30000L);
			break;
		case HSM_ON_EXIT : 
			printf("Next one, please...\n");
			break;
		case TIMER_TIMEOUT_T2 :
			HsmSetNewState(HSM_ID_Hkbd,&HkbdState_WaitingActivation);
			break;
	}	
}

// END OF FILE



