//============================================================================
// A simplified Hierarchical state machine demo
//
// (c) Robert Lacoste 2013, 
// Based on ideas from Miro Samek's book "Practival UML statecharts in C/C++)"
//
// Use this software freely, but provided without any warranty or support
//
// Version for Dev-C++ / PC
//============================================================================

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <time.h> 

#include "HSM.h"
#include "Hkbd.h"
#include "Lowlevel.h"


//----------------------------------------------------------------------------
// Idle manager
//
// This function is called whenever there is no more message to manage
// in any HSM
//
// This is a polled simulation, usually the idle manager should switch
// the CPU to low power mode and all events should be generated trough
// interrupt routines
//----------------------------------------------------------------------------

static time_t timelastsecond;

void IdleManager(void)
{
	BOOL anyevent=FALSE;
	char c;
	time_t newtime;
	
	do
	{
		//check keys and generate corresponding event
		if(kbhit())
		{
			c=(char)getch();
			switch(c)
			{
				case '*' :
					HsmSendMsg(HSM_ID_Hkbd,Hkbd_KEY_ACTIVATE,0);
					anyevent=TRUE;
					break;
				case '0' :
				case '1' :
				case '2' :
				case '3' :
				case '4' :
				case '5' :
				case '6' :
				case '7' :
				case '8' :
				case '9' :
					HsmSendMsg(HSM_ID_Hkbd,Hkbd_KEY_DIGIT,c-'0');
					anyevent=TRUE;
					break;
				case '/' :
					HsmSendMsg(HSM_ID_Hkbd,Hkbd_KEY_ALPHAMODE,0);
					anyevent=TRUE;
					break;
				case '-' :
					HsmSendMsg(HSM_ID_Hkbd,Hkbd_KEY_DEL,0);
					anyevent=TRUE;
					break;
				case (char)13 :
					HsmSendMsg(HSM_ID_Hkbd,Hkbd_KEY_ENTER,0);
					anyevent=TRUE;
					break;
			}
		}
				
		// Check time and update timers if one second elapsed
		time(&newtime); 
    	if(difftime(newtime,timelastsecond)>=1.0) 
		{
			putch('.');
			if(TimerTick1s())
				anyevent=TRUE;
			timelastsecond=newtime;
		}
	}
	while(!anyevent);
}


//----------------------------------------------------------------------------
// Main program
//----------------------------------------------------------------------------

int main(int argc, char *argv[]) 
{
	// Initialise dmo
	printf("FSM_demo started\n");
	time(&timelastsecond); 
	
	// Initialise modules
	TimerInit();
	HsmInit();
	HkbdInit();
	HcmdInit();
	
	// Define the idle function
	HsmDefineIdleFunction(&IdleManager);
	
	// And launch the state machines manager
	HsmRun();
	
	// Should never come here
	return 0;
}

// END OF FILE

